unit ClipMonitor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, clipbrd;

type
  TClipFormat = (cfNone, cfText, cfBitmap, cfMetaFilePict,
    cfPicture, cfComponent, cfOther);

  TClipMon = class(TComponent)
  private
    FHandle, FNextViewerHandle: THandle;
    FActive: Boolean;
    FOnChange: TNotifyEvent;
    OldWndProc, NewWndProc: Pointer;
    procedure SetActive(Value: Boolean);
    function GetFormat: TClipFormat;
    procedure SetFormat(Value: TClipFormat);
  protected
    procedure NewWndMethod(var msg: TMessage);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Active: Boolean read FActive write SetActive default False;
    property Format: TClipFormat read GetFormat write SetFormat stored False;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Flyup', [TClipMon]);
end;

constructor TClipMon.Create(AOwner: TComponent);
begin
  if not (AOwner is TForm) then
    raise Exception.CreateFmt('''%s'' is not a Form', [AOwner.Name])
  else if AOwner = nil then
    raise Exception.Create('No Windowized Owner');
  inherited Create(AOwner);
  FHandle := (Owner as TForm).Handle;
  FNextViewerHandle := 0;
  FActive := False;
  FNextViewerHandle := SetClipboardViewer(FHandle);
  if not (csDesigning in ComponentState) then begin
      NewWndProc := MakeObjectInstance(NewWndMethod);
      OldWndProc := Pointer(SetWindowLong(FHandle, GWL_WNDPROC,
        LongInt(NewWndProc)));
    end else begin
      NewWndProc := nil;
      OldWndProc := nil;
    end;
end;

destructor TClipMon.Destroy;
begin
  ChangeClipboardChain(FHandle, FNextViewerHandle);
  if Assigned(NewWndProc) then begin
      SetWindowLong(FHandle, GWL_WNDPROC, Longint(OldWndProc));
      FreeObjectInstance(NewWndProc);
    end;
  inherited Destroy;
end;

{]mO_\۰ʰŶKOʱ}

procedure TClipMon.SetActive(Value: Boolean);
begin
  if FActive <> Value then FActive := Value;
end;

{ŶKO榡}

function TClipMon.GetFormat: TClipFormat;
var
  I: Integer;
begin
  Result := cfNone;
  if Clipboard.HasFormat(CF_TEXT) then Result := cfText
  else if Clipboard.HasFormat(CF_BITMAP) then Result := cfBitmap
  else if Clipboard.HasFormat(CF_METAFILEPICT) then Result := cfMetaFilePict
  else if Clipboard.HasFormat(CF_PICTURE) then Result := cfPicture
  else if Clipboard.HasFormat(CF_COMPONENT) then Result := cfComponent
  else for I := 0 to Clipboard.FormatCount - 1 do
      if Clipboard.HasFormat(Clipboard.Formats[I]) then begin
          Result := cfOther;
          Break;
        end;
end;

{{ǡAƹOŶKO榡OuŪ}

procedure TClipMon.SetFormat(Value: TClipFormat);
begin
end;

{BzӦ۰ŶKO[쪺}

procedure TClipMon.NewWndMethod(var msg: TMessage);
begin
  case msg.msg of
    WM_DRAWCLIPBOARD:
      begin
        Loaded;
        msg.Result := SendMessage(WM_DRAWCLIPBOARD,
          FNextViewerHandle, 0, 0);
      end;
    WM_CHANGECBCHAIN:
      if THandle(msg.wParam) = FNextViewerHandle then begin
          FNextViewerHandle := msg.lParam;
          msg.Result := 0;
        end else
        msg.Result := SendMessage(FNextViewerHandle, WM_CHANGECBCHAIN,
          msg.wParam, msg.lParam);
    else
      msg.result := CallWindowProc(OldWndProc, FHandle, msg.msg,
        msg.WParam, msg.LParam);
  end;
end;

{˸}

procedure TClipMon.Loaded;
begin
  if FActive and Assigned(FOnChange) then FOnChange(Self);
end;

end.

